#include <OneWire.h>
#include <DallasTemperature.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define Buzzer_Pin 12
#define SENSOR_BUS_PIN 3  // I sensori di temperatura DS18D20 sono su D12

OneWire oneWire(SENSOR_BUS_PIN);      //Creiamo istanza OneWire per utilizzare il canale di comunicazione
DallasTemperature sensors(&oneWire);  //Passiamo il canale di comunicazione alla libreria della Dallas

Adafruit_SSD1306 display(128, 32, &Wire, 4);
#define SCREEN_WIDTH 128  // OLED display width, in pixels
#define SCREEN_HEIGHT 32  // OLED display height, in pixels
int numSensori = 0;
int mintemp = 51;    //temperatura minima
int midtemp = 52;    // temperatura a cui si spegne la resistenza 2
int maxtemp = 53;    //temperatura massima. Si spegne la resistenza 1
int alarmtemp = 80;  //temperatura critica: attivazione allarme, spegnimento relays
int keyVal;          // costante per il tasto di switch


void setup() {
  Serial.begin(9600);  // si attiva il monitor seriale
  Serial.println("DS18B20 test");
  sensors.begin();  //Inizializzo comunicazione

  pinMode(10, OUTPUT);  //led alimentazione  D4
  pinMode(9, OUTPUT);   //relay 1 + led rosso di riscaldamento  D9
  pinMode(11, OUTPUT);  //relay 2 + led rosso di riscaldamento  D11
  pinMode(13, OUTPUT);  //cicalino allarme   D13

  //Scansione sensori sul bus sensori DS18B20
  Serial.print("Scansione dispositivi... ");
  Serial.print("Trovati ");
  numSensori = sensors.getDeviceCount();
  Serial.print(numSensori, DEC);
  Serial.println(" sensori.");
  Serial.println("");

  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {  //inizializzazione display
    while (true)
      ;
  }
  display.display();
  display.clearDisplay();
}

void loop() {
  keyVal = analogRead(A2);  //configuro il pulsante di switch per la temp. sul display
  Serial.print("Richiesta temperatura... ");
  sensors.requestTemperatures();  //Invio comando per leggere temperatura
  Serial.println("OK");
  //Stampo temperatura del sensore i-esimo (nel nostro caso sensore 0 e sensore 1
  //  for(int i=0;i<numSensori;i++){
  printSensor(0);
  printSensor(1);
  //  }
}

void printSensor(int i) {
  Serial.print("Sensore ");  //stampa la temperatura dei sensori sul monitor seriale
  Serial.print(i + 1);
  Serial.print(": ");
  float tempC = sensors.getTempCByIndex(i);
  float tempC0 = sensors.getTempCByIndex(0);  //sensore 0: temperatura del liquido
  float tempC1 = sensors.getTempCByIndex(1);  //sensore 1: temperatura della resistenza
  Serial.print(tempC);
  Serial.print("°C | ");
  Serial.print(DallasTemperature::toFahrenheit(tempC));
  Serial.println("°F");

  display.clearDisplay();       //stampa la temperatura sul display OLED
  display.setTextSize(2);       //stampa la temperatura del sensore zero, tranne:
  display.setTextColor(WHITE);  // 1: si preme il tastino di switch
  display.cp437(true);          // 2: la temperatura raggiunge la soglia di allarme
  display.setCursor(0, 0);
  display.print(tempC0);
  display.print(" C");
  display.display();

  if (keyVal > 1000) {  //se si preme il tasto di switch, mostra la temperaura del sensore della resistenza
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(WHITE);
    display.cp437(true);
    display.setCursor(0, 0);
    display.print(tempC1);
    display.print(" C");
    display.display();
  }
  if (tempC0 < mintemp) {    // se la temperatura è bassa, si accende il relay della resistenza 1
    digitalWrite(9, HIGH);   // si attiva il relay 1 + il led di riscaldamento (rosso)
    digitalWrite(10, HIGH);  //si accende il led alimentazione
    noTone(Buzzer_Pin);      //il cicalino è spento
    display.display();       //sul display appare la scritta
    display.setCursor(0, 16);
    display.print("Warming up");
    display.display();
  } else if (tempC0 > midtemp && tempC0 <= maxtemp) {  //se la temperatura ha raggiunto il massimo, i relays si spengono
    digitalWrite(9, LOW);                              //si disattiva il relay 1 e si spegne il led di riscaldamento
    digitalWrite(11, LOW);                             //si disattiva il relay 2
    digitalWrite(10, HIGH);                            //resta acceso il led alimentazione
    display.setCursor(0, 16);                          //sul display appare la scritta
    display.print("Temp. OK!");
    display.display();
    noTone(Buzzer_Pin);  //il cicalino è spento
  }

  if (tempC0 > mintemp && tempC0 <= midtemp) {
    display.setCursor(0, 16);  //sul display appare la scritta
    display.print("Temp. OK!");
    display.display();
  }

  if (tempC0 < mintemp) {
    digitalWrite(11, HIGH);  //quando la temperatura è prossima a quella ottimale,si disattiva il relay 2
  } else if (tempC0 > mintemp) {
    digitalWrite(11, LOW);
  }

  if (tempC0 >= mintemp && tempC0 <= maxtemp) {  //se la temperatura è tra il minimo e il massimo,
    display.setCursor(0, 16);
    display.print("Temp. OK!");  //sul display appare la scritta
    display.display();
    digitalWrite(10, HIGH);  //resta acceso il led alimentazione
    digitalWrite(9, LOW);    //si spegne il led riscaldamento
  }

  if (tempC1 >= alarmtemp) {  // se la temperatura delle resistenze raggiunge il massimo consentito, si attiva l'allarme
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(WHITE);
    display.cp437(true);
    display.setCursor(0, 0);
    display.print(tempC1);
    display.print(" C");
    display.display();
    digitalWrite(9, LOW);    //si disattiva il relay 1 del riscaldamento, se fosse ancora acceso
    digitalWrite(11, LOW);   //si disattiva il relay 2 del riscaldamento, se fosse ancora acceso
    digitalWrite(10, HIGH);  //il led verde lampeggia
    delay(100);
    digitalWrite(11, HIGH);
    delay(100);
    tone(Buzzer_Pin, 2000, 1000);  //il cicalino emette una nota di allarme
    display.setCursor(0, 16);
    display.print("HIGH TEMP");  //sul display appare la scritta
    display.display();
  }
}
